@@ -1,12 +1,11 @@
+from __future__ import annotations
+
 import contextlib
 import functools
 import os
 import sys
-from typing import Dict
 from typing import Generator
-from typing import Optional
 from typing import Sequence
-from typing import Tuple
 
 import pre_commit.constants as C
 from pre_commit.envcontext import envcontext
@@ -21,6 +20,7 @@ from pre_commit.util import CalledProcessError
 from pre_commit.util import clean_path_on_failure
 from pre_commit.util import cmd_output
 from pre_commit.util import cmd_output_b
+from pre_commit.util import win_exe
 
 ENVIRONMENT_DIR = 'py_env'
 
@@ -34,9 +34,9 @@ def _version_info(exe: str) -> str:
         return f'<<error retrieving version from {exe}>>'
 
 
-def _read_pyvenv_cfg(filename: str) -> Dict[str, str]:
+def _read_pyvenv_cfg(filename: str) -> dict[str, str]:
     ret = {}
-    with open(filename) as f:
+    with open(filename, encoding='UTF-8') as f:
         for line in f:
             try:
                 k, v = line.split('=')
@@ -64,7 +64,7 @@ def get_env_patch(venv: str) -> PatchesT:
 
 def _find_by_py_launcher(
         version: str,
-) -> Optional[str]:  # pragma: no cover (windows only)
+) -> str | None:  # pragma: no cover (windows only)
     if version.startswith('python'):
         num = version[len('python'):]
         cmd = ('py', f'-{num}', '-c', 'import sys; print(sys.executable)')
@@ -76,8 +76,8 @@ def _find_by_py_launcher(
     return None
 
 
-def _find_by_sys_executable() -> Optional[str]:
-    def _norm(path: str) -> Optional[str]:
+def _find_by_sys_executable() -> str | None:
+    def _norm(path: str) -> str | None:
         _, exe = os.path.split(path.lower())
         exe, _, _ = exe.partition('.exe')
         if exe not in {'python', 'pythonw'} and find_executable(exe):
@@ -132,7 +132,7 @@ def _sys_executable_matches(version: str) -> bool:
     return sys.version_info[:len(info)] == info
 
 
-def norm_version(version: str) -> Optional[str]:
+def norm_version(version: str) -> str | None:
     if version == C.DEFAULT:  # use virtualenv's default
         return None
     elif _sys_executable_matches(version):  # virtualenv defaults to our exe
@@ -163,27 +163,44 @@ def in_env(
         yield
 
 
-def healthy(prefix: Prefix, language_version: str) -> bool:
+def health_check(prefix: Prefix, language_version: str) -> str | None:
     directory = helpers.environment_dir(ENVIRONMENT_DIR, language_version)
     envdir = prefix.path(directory)
     pyvenv_cfg = os.path.join(envdir, 'pyvenv.cfg')
 
     # created with "old" virtualenv
     if not os.path.exists(pyvenv_cfg):
-        return False
+        return 'pyvenv.cfg does not exist (old virtualenv?)'
 
-    exe_name = 'python.exe' if sys.platform == 'win32' else 'python'
+    exe_name = win_exe('python')
     py_exe = prefix.path(bin_dir(envdir), exe_name)
     cfg = _read_pyvenv_cfg(pyvenv_cfg)
 
-    return (
-        'version_info' in cfg and
-        # always use uncached lookup here in case we replaced an unhealthy env
-        _version_info.__wrapped__(py_exe) == cfg['version_info'] and (
-            'base-executable' not in cfg or
-            _version_info(cfg['base-executable']) == cfg['version_info']
+    if 'version_info' not in cfg:
+        return "created virtualenv's pyvenv.cfg is missing `version_info`"
+
+    # always use uncached lookup here in case we replaced an unhealthy env
+    virtualenv_version = _version_info.__wrapped__(py_exe)
+    if virtualenv_version != cfg['version_info']:
+        return (
+            f'virtualenv python version did not match created version:\n'
+            f'- actual version: {virtualenv_version}\n'
+            f'- expected version: {cfg["version_info"]}\n'
         )
-    )
+
+    # made with an older version of virtualenv? skip `base-executable` check
+    if 'base-executable' not in cfg:
+        return None
+
+    base_exe_version = _version_info(cfg['base-executable'])
+    if base_exe_version != cfg['version_info']:
+        return (
+            f'base executable python version does not match created version:\n'
+            f'- base-executable version: {base_exe_version}\n'
+            f'- expected version: {cfg["version_info"]}\n'
+        )
+    else:
+        return None
 
 
 def install_environment(
@@ -208,6 +225,6 @@ def run_hook(
         hook: Hook,
         file_args: Sequence[str],
         color: bool,
-) -> Tuple[int, bytes]:
+) -> tuple[int, bytes]:
     with in_env(hook.prefix, hook.language_version):
         return helpers.run_xargs(hook, hook.cmd, file_args, color=color)
